/* small program to preprocess data for inclusion in cairo_snippets
 * (c) Øyvind Kolås, placed in the public domain
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_SNIPPETS 512

char *snippet_name[MAX_SNIPPETS];
int   snippet_count=0;

static FILE *out;

static void
make_header (void)
{
    fprintf (out, "/* DO NOT EDIT!, file autogenerated by prepare_snippets, using *.cairo files */\n");
    fprintf (out, "#include \"snippets.h\"\n");
}

static void
make_index (void)
{
    int i;

    fprintf (out, "\n\nint snippet_count=%i;\n", snippet_count);
    fprintf (out, "\n");
    fprintf (out, "const char *snippet_name[%i]={\n", snippet_count+1);

    for (i=0;i<snippet_count;i++){
        fprintf (out, "\"%s\",\n", snippet_name[i]);
    }
    fprintf (out, "NULL\n");
    fprintf (out, "};\n");
}

static void
make_do_snippet (void)
{
    int i;

    fprintf (out, "\n\nvoid\n"
                  "snippet_do (cairo_t *cr, int snippet_no)\n"
                  "{\n"
                  "   switch (snippet_no) {\n");

    for (i=0;i<snippet_count;i++){
       char filename[40];
       char c;
       int  read;

       FILE *file;

       fprintf (out, "\n");
       fprintf (out, "     case %i:{ /* %s */\n\n\n", i, snippet_name[i]);

       sprintf (filename, "%s.cairo", snippet_name[i]);
       file = fopen (filename, "r");

       while ((read=fread (&c, 1,1, file))){
           fprintf (out, "%c", c);

       }

       fclose (file);

       fprintf (out, "\n\n             }break;\n");
    }

    fprintf (out, "     default:\n"
                  "       fprintf (stderr, \"do_snippet: snippet_no %%i, out of range\\n\", snippet_no);\n");
    fprintf (out, "   }\n"
                  "}\n");
}

static void
make_snippet_source (void)
{
    int i;
    fprintf (out, "\n\nconst char *snippet_source[%i]={\n", snippet_count+1);

    for (i=0;i<snippet_count;i++){
       char filename[40];
       char c;
       int  read;

       FILE *file;

       fprintf (out, "\"");

       sprintf (filename, "%s.cairo", snippet_name[i]);
       file = fopen (filename, "r");

       while ((read=fread (&c, 1,1, file))){
           switch (c){
                   case '\"':
                      fprintf (out, "\\\"");
                      break;
                   case '\n':
                      fprintf (out, "\\n\"\n\"");
                      break;
                   case '\\':
                      fprintf (out, "\\\\");
                      break;
                   default:
                      fprintf (out, "%c", c);
           }
       }
       fprintf (out, "\",\n");

       fclose (file);
    }
    fprintf (out, "NULL\n");
    fprintf (out, "};\n");
}

static void
make_util (void)
{
    fprintf (out, 
"int\n"
"snippet_name2no (const char *name)\n"
"{\n"
"        int i;\n"
"        for (i=0;i<snippet_count;i++)\n"
"                if (!strcmp(name, snippet_name[i]))\n"
"                        return i;\n"
"        fprintf (stderr, \"snippet_name2no: by name '%%s' not found\\n\", name);\n"
"        return -1;\n"
"}\n");


    fprintf (out,
"void\n"
"snippet_set_bg_png (cairo_t *cr, const char *file)\n"
"{\n"
"   int w,h,stride;\n"
"   char *buffer;\n"
"   cairo_surface_t *image;\n"
"   buffer = read_png_argb32 (file, &w,&h, &stride);\n"
"   image = cairo_surface_create_for_image (buffer, CAIRO_FORMAT_ARGB32,w,h,stride);\n"
"   cairo_save (cr);\n"
"       cairo_scale (cr, 1.0/w, 1.0/h);\n"
"       cairo_move_to (cr, 0,0);\n"
"       cairo_show_surface (cr, image,w,h);\n"
"   cairo_restore (cr);\n"
"   cairo_surface_destroy (image);\n"
"   free (buffer);\n"
"}\n"
);

    fprintf (out,
"void\n"
"snippet_set_bg_svg (cairo_t *cr, const char *file)\n"
"{\n"
"   int width,height;\n"
"   svg_cairo_t *svgc;\n"
"   svg_cairo_create (&svgc);\n"
"   svg_cairo_parse (svgc, file);\n"
"   svg_cairo_get_size (svgc, &width, &height);\n"
"   cairo_save (cr);\n"
"     cairo_scale (cr, 1.0/width, 1.0/height);\n"
"     svg_cairo_render (svgc, cr);\n"
"   cairo_restore (cr);\n"
"   svg_cairo_destroy (svgc);\n"
"}\n"
);

}

int
main (int argc, char **argv)
{
    char **cur_arg;

    out   = fopen ("snippets.c", "w");

    cur_arg=&argv[1];
    while (*cur_arg) {
            if (*cur_arg[0]!='-'){
                char *name = strdup (*cur_arg);
                FILE *snippet = fopen (name, "r");
                if (!strstr (name, ".cairo") || !snippet) {
                        fprintf (stderr, "unable to open '%s' or snippet name doesn't end in .cairo\n", name);
                        exit (-1);
                }

                if (name) {
                    *strstr (name, ".cairo")='\0';
                    snippet_name[snippet_count++]=strdup (name);
                    free (name);
                }
            }
            cur_arg++;
    }

    make_header ();
    make_index ();
    make_snippet_source ();
    make_do_snippet ();
    make_util ();

    fclose (out);

    return 0;
}
