/* 
 * cairo-gtk-demo: A simple demo showing how to draw a Gtk+ widget with cairo.
 *
 * Copyright (C) 2002, Amaury Jacquot
 *
 * Amaury Jacquot <sxpert@esitcom.org>
 * Carl Worth <cworth@isi.edu>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "cairo-widget.h"

typedef struct _CairoGtkDemo {
    GtkWidget *window;
    CairoWidget *cairo_widget;
} CairoGtkDemo;

static void
destroy (GtkWidget* widget, gpointer data)
{
    CairoGtkDemo *demo = data;

    demo->window = NULL;
    gtk_main_quit ();
}

static void
key_press (GtkWidget *widget, GdkEventKey *event, CairoGtkDemo *demo)
{
    switch (event->keyval) {
    case GDK_Q:
    case GDK_q:
	gtk_main_quit ();
	break;
    default:
	cairo_widget_randomize (demo->cairo_widget);
	break;
    }
}

int
main (int argc,char* argv[])
{
    CairoGtkDemo demo;

    gtk_init (&argc, &argv);

    demo.window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    demo.cairo_widget = CAIRO_WIDGET (cairo_widget_new ());

    g_signal_connect (G_OBJECT (demo.window), "destroy",
		      G_CALLBACK (destroy), &demo);

    g_signal_connect (G_OBJECT (demo.window), "key_press_event",
		      G_CALLBACK (key_press), &demo);
	
    gtk_container_add (GTK_CONTAINER (demo.window), GTK_WIDGET (demo.cairo_widget));
    
    gtk_widget_show_all (demo.window);
	
    gtk_main();

    return 0;
}
